@@ -1,26 +1,26 @@
-"""some various graph manipuliation utilities
+"""Graph manipuliation utilities.
 
 (dot generation adapted from pypy/translator/tool/make_dot.py)
 
-:organization: Logilab
-:copyright: 2003-2007 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
+:copyright: 2000-2008 LOGILAB S.A. (Paris, FRANCE), all rights reserved.
 :contact: http://www.logilab.fr/ -- mailto:contact@logilab.fr
+:license: General Public License version 2 - http://www.gnu.org/licenses
 """
-
 __docformat__ = "restructuredtext en"
+
 __metaclass__ = type
 
 import os.path as osp
 import os
 
 def escape(value):
-    """make <value> usable in a dot file"""
+    """Make <value> usable in a dot file."""
     lines = [line.replace('"', '\\"') for line in value.split('\n')]
     data = '\\l'.join(lines)
     return '\\n' + data
 
 def target_info_from_filename(filename):
-    """transforms /some/path/foo.png into ('/some/path', 'foo.png', 'png')"""
+    """Transforms /some/path/foo.png into ('/some/path', 'foo.png', 'png')."""
     abspath = osp.abspath(filename)
     basename = osp.basename(filename)
     storedir = osp.dirname(abspath)
@@ -29,9 +29,11 @@ def target_info_from_filename(filename):
 
 
 class DotBackend:
-    """Dot File backend"""
-    def __init__(self, graphname, rankdir=None, size=None, ratio=None, charset='utf-8'):
+    """Dot File backend."""
+    def __init__(self, graphname, rankdir=None, size=None, ratio=None,
+            charset='utf-8', renderer='dot', additionnal_param={}):
         self.graphname = graphname
+        self.renderer = renderer
         self.lines = []
         self._source = None
         self.emit("digraph %s {" % normalize_node_id(graphname))
@@ -45,6 +47,8 @@ class DotBackend:
             assert charset.lower() in ('utf-8', 'iso-8859-1', 'latin1'), \
                    'unsupported charset %s' % charset
             self.emit('charset="%s"' % charset)
+        for param in additionnal_param.iteritems():
+            self.emit('='.join(param))
 
     def get_source(self):
         """returns self._source"""
@@ -57,9 +61,12 @@ class DotBackend:
     source = property(get_source)
     
     def generate(self, outputfile=None, dotfile=None):
-        """generates a graph file
-        :param target: output format ('png', 'ps', etc.). If None,
-                       the raw dot source will be returned
+        """Generates a graph file.
+        
+        :param outputfile: filename and path [defaults to graphname.png]
+        :param dotfile: filename and path [defaults to graphname.dot]
+        
+        :rtype: str
         :return: a path to the generated file
         """
         if outputfile is not None:
@@ -78,31 +85,31 @@ class DotBackend:
             pdot.write(self.source)
         pdot.close()
         if target != 'dot':
-            os.system('dot -T%s %s -o%s' % (target, dot_sourcepath, outputfile))
+            os.system('%s -T%s %s -o%s' % (self.renderer, target, dot_sourcepath, outputfile))
             os.unlink(dot_sourcepath)
         return outputfile
 
     def emit(self, line):
-        """adds <line> to final output"""
+        """Adds <line> to final output."""
         self.lines.append(line)
 
     def emit_edge(self, name1, name2, **props):
-        """emits edge from <name1> to <name2>
+        """Emits edge from <name1> to <name2>.
         
-        authorized props: see http://www.graphviz.org/doc/info/attrs.html
+        Authorized props: see http://www.graphviz.org/doc/info/attrs.html
         """
         attrs = ['%s="%s"' % (prop, value) for prop, value in props.items()]
         self.emit('edge [%s];' % ", ".join(attrs))
         self.emit('%s -> %s' % (normalize_node_id(name1), normalize_node_id(name2)))
 
     def emit_node(self, name, **props):
-        """authorized props: see http://www.graphviz.org/doc/info/attrs.html
+        """Authorized props: see http://www.graphviz.org/doc/info/attrs.html
         """
         attrs = ['%s="%s"' % (prop, value) for prop, value in props.items()]
         self.emit('%s [%s];' % (normalize_node_id(name), ", ".join(attrs)))
 
 def normalize_node_id(nid):
-    """returns a suitable DOT node id for `nid`"""
+    """Returns a suitable DOT node id for `nid`."""
     return '"%s"' % nid
 
 class GraphGenerator:
